<?php

namespace App\Controller;

use App\Gateway\AuthGateway;
use App\Http\HttpRequest;
use App\Http\HttpResponse;
use App\Http\ViewHttpResponse;
use App\Model\AuthModel;
use App\Validation\FieldValidationFail;
use App\Validation\Validators;
use Twig\Environment;


class AuthController {
    private AuthModel $model;

    /**
     * @param AuthModel $model
     */
    public function __construct(AuthModel $model) {
        $this->model = $model;
    }

    public function displayRegister(): HttpResponse {
        return ViewHttpResponse::twig("display_register.html.twig", []);
    }

    private function displayRegisterBadFields(array $fails): HttpResponse{
        $bad_fields = [];
        foreach ($fails as $err) {
            if ($err instanceof FieldValidationFail) {
                $bad_fields[] = $err->getFieldName();
            }
        }
        return ViewHttpResponse::twig("display_register.html.twig", ['bad_fields' => $bad_fields]);
    }
    public function confirmRegister(array $request): HttpResponse {

        $fails = [];
        $request = HttpRequest::from($request, $fails, [
            "username" => [Validators::name(), Validators::lenBetween(0, 32)],
            "password" => [Validators::lenBetween(6, 256)],
            "confirmpassword" => [Validators::lenBetween(6, 256)],
            "email" => [Validators::regex("/^[\w-\.]+@([\w-]+\.)+[\w-]{2,4}$/g"),Validators::lenBetween(5, 256)]
        ]);


        if (!empty($fails)) {
            return $this->displayRegisterBadFields($fails);
        }

        $fails = $this->model->validationRegister($request['username'], $request["password"], $request['confirmpassword'], $request['email']);
        if (empty($fails)) {
            $results = $this->model->getUserFields($request['email']);
            return ViewHttpResponse::twig("display_auth_confirm.html.twig", ['username' => $results['username'], 'email' => $results['email']]);
        }

        return $this->displayRegisterBadFields($fails);
    }

    public function displayLogin():HttpResponse{
        return ViewHttpResponse::twig("display_login.html.twig", []);
    }

    public function displayLoginPassed(array $request):HttpResponse{
        $fails = [];
        $request = HttpRequest::from($request, $fails, [
            "password" => [Validators::lenBetween(0, 256)],
            "email" => [Validators::regex("/@/")]
        ]);


        $fails = $this->model->validationLogin($request['email'],$request['password']);

        if (empty($fails)){
            $results = $this->model->getUserFields($request['email']);
            return ViewHttpResponse::twig("display_auth_confirm.html.twig",['username' => $results['username'], 'email' => $results['email']]);
        }
        return $this->displayRegisterBadFields($fails);
    }

}